
/*
 * Apple II ProDOS file types as of June, 1992
 *
 * See "About Apple II File Type Notes" by Apple Developer Technical
 * Support for more information.
 * 
 * This list is taken directly from the above named document, with the
 * exception of the file types $B6 and $F9, for which the abbreviations
 * "str" and "p16" are often found alongside the official abbreviations
 * ("pif" and "os" respectively).
 */

struct ftypes {
	unsigned char code;
	char name[4];
} filetypes[] = {
	0x00, "non", /* Unknown */
	0x01, "bad", /* Bad blocks */
	0x02, "pcd", /* Pascal code */
	0x03, "ptx", /* Pascal text */
	0x04, "txt", /* ASCII text */
	0x05, "pda", /* Pascal data */
	0x06, "bin", /* Binary */
	0x07, "fnt", /* Apple /// Font */
	0x08, "fot", /* Apple II or /// Graphics */
	0x09, "ba3", /* Apple /// BASIC program */
	0x0a, "da3", /* Apple /// BASIC data */
	0x0b, "wpf", /* Apple II or /// Word Processor */
	0x0c, "sos", /* Apple /// SOS System */
	0x0f, "dir", /* Folder */
	0x10, "rpd", /* Apple /// RPS data */
	0x11, "rpi", /* Apple /// RPS index */
	0x12, "afd", /* Apple /// AppleFile discard */
	0x13, "afm", /* Apple /// AppleFile model */
	0x14, "afr", /* Apple /// AppleFile report format */
	0x15, "scl", /* Apple /// screen library */
	0x16, "pfs", /* PFS document */
	0x19, "adb", /* AppleWorks Data Base */
	0x1a, "awp", /* AppleWorks Word Processor */
	0x1b, "asp", /* AppleWorks Spreadsheet */
	0x20, "tdm", /* Desktop Manager document */
	0x2a, "8sc", /* Apple II Source Code */
	0x2b, "8ob", /* Apple II Object Code */
	0x2c, "8ic", /* Apple II Interpreted Code */
	0x2d, "8ld", /* Apple II Language Data */
	0x2e, "p8c", /* ProDOS 8 code module */
	0x40, "dic", /* Dictionary file */
	0x42, "ftd", /* File Type Names */
	0x50, "gwp", /* Apple IIgs Word Processor */
	0x51, "gss", /* Apple IIgs Spreadsheet */
	0x52, "gdb", /* Apple IIgs Data Base */
	0x53, "drw", /* Drawing */
	0x54, "gdp", /* Desktop Publishing */
	0x55, "hmd", /* Hypermedia */
	0x56, "edu", /* Educational Data */
	0x57, "stn", /* Stationery */
	0x58, "hlp", /* Help File */
	0x59, "com", /* Communications File */
	0x5a, "cfg", /* Configuration file */
	0x5b, "anm", /* Animation file */
	0x5c, "mum", /* Multimedia document */
	0x5d, "ent", /* Game/Entertainment document */
	0x5e, "dvu", /* Development utility document */
	0x5f, "fin", /* Financial document */
	0x6b, "bio", /* PC Transporter BIOS */
	0x6d, "tdr", /* PC Transporter driver */
	0x6e, "pre", /* PC Transporter pre-boot */
	0x6f, "hdv", /* PC Transporter volume */
	0xa0, "wp",  /* WordPerfect document */
	0xab, "gsb", /* Apple IIgs BASIC program */
	0xac, "tdf", /* Apple IIgs BASIC TDF */
	0xad, "bdf", /* Apple IIgs BASIC data */
	0xb0, "src", /* Apple IIgs  source code */
	0xb1, "obj", /* Apple IIgs object code */
	0xb2, "lib", /* Apple IIgs Library file */
	0xb3, "s16", /* GS/OS application */
	0xb4, "rtl", /* GS/OS Run-Time Library */
	0xb5, "exe", /* GS/OS Shell application */
	0xb6, "pif", /* Permanent initialization file */
	0xb6, "str", /* Permanent initialization file (alternate) */
	0xb7, "tif", /* Temporary initialization file */
	0xb8, "nda", /* New desk accessory */
	0xb9, "cda", /* Classic desk accessory */
	0xba, "tol", /* Tool */
	0xbb, "dvr", /* Apple IIgs Device Driver file */
	0xbc, "ldf", /* Load file (generic) */
	0xbd, "fst", /* GS/OS File System Translator */
	0xbf, "doc", /* GS/OS document */
	0xc0, "pnt", /* Packed Super Hi-Res picture */
	0xc1, "pic", /* Super Hi-Res picture */
	0xc2, "ani", /* Paintworks animation */
	0xc3, "pal", /* Paintworks palette */
	0xc5, "oog", /* Object-oriented graphics */
	0xc6, "scr", /* Script */
	0xc7, "cdv", /* Control Panel document */
	0xc8, "fon", /* Font */
	0xc9, "fnd", /* Finder data */
	0xca, "icn", /* Icons */
	0xd5, "mus", /* Music sequence */
	0xd6, "ins", /* Instrument */
	0xd7, "mdi", /* MIDI data */
	0xd8, "snd", /* Sampled sound */
	0xdb, "dbm", /* DB Master document */
	0xe0, "lbr", /* Archival library */
	0xe2, "atk", /* AppleTalk data */
	0xee, "r16", /* EDASM 816 relocatable file */
	0xef, "pas", /* Pascal area */
	0xf0, "cmd", /* BASIC command */
	0xf9, "os",  /* GS/OS System file */
	0xf9, "p16", /* GS/OS System file (alternate) */
	0xfa, "int", /* Integer BASIC program */
	0xfb, "ivr", /* Integer BASIC variables */
	0xfc, "bas", /* AppleSoft BASIC program */
	0xfd, "var", /* AppleSoft BASIC variables */
	0xfe, "rel", /* Relocatable code */
	0xff, "sys"  /* ProDOS 8 application */
};

#define FTYPESIZE sizeof(filetypes)/sizeof(struct ftypes)
